#!/usr/bin/env python3
# To the extent possible under law, the author has dedicated all
# copyright and related and neighboring rights to this software to the
# public domain worldwide.
# THIS SOFTWARE IS DISTRIBUTED WITHOUT ANY WARRANTY.
import argparse
import csv
from pathlib import Path
import sys

import imapclient


def write_csv(client, fname):
    if fname.exists():
        print(f"{fname} already exists.")
        return False
    client.select_folder("INBOX", readonly=True)
    messages = client.search("ALL")
    response = client.fetch(messages, ["ENVELOPE"])
    with fname.open("w") as f:
        writer = csv.writer(f)
        for message_id, data in response.items():
            env = data[b"ENVELOPE"]
            from_ = (
                env.from_[0].mailbox.decode("utf-8")
                + "@"
                + env.from_[0].host.decode("utf-8")
            )
            subj = env.subject.decode("utf-8") if env.subject else ""
            writer.writerow(["", message_id, from_, subj])
    return True


def read_csv(client, fname):
    if not fname.exists():
        print(f"{fname} does not exist.")
        return False
    to_delete = []
    with fname.open() as f:
        reader = csv.DictReader(f, ["delete", "id", "from", "subject"])
        for row in reader:
            if row["delete"] == "x":
                to_delete.append(row["id"])
    client.select_folder("INBOX")
    trash = None
    for folder in client.list_folders():
        if len(folder[0]) and folder[0][0] == b"\\Trash":
            trash = folder[2]
            break
    if not trash:
        print("Trash not found!")
        return False
    client.move(to_delete, trash)
    return True


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("action", choices=["write-csv", "read-csv"])
    parser.add_argument("filename", type=Path)
    parser.add_argument("--host", required=True)
    parser.add_argument("--username", required=True)
    parser.add_argument("--password", required=True)
    parser.add_argument(
        "--starttls",
        action="store_true",
        default=False,
        help="Start the connection as plain text and try to use the StartTLS command. "
        "If not specified, the connection will use TLS on the port 993.",
    )
    args = parser.parse_args()

    with imapclient.IMAPClient(
        host=args.host, ssl=not args.starttls
    ) as client:
        if args.starttls:
            client.starttls()
        client.login(args.username, args.password)
        if args.action == "read-csv":
            rv = read_csv(client, args.filename)
        else:
            rv = write_csv(client, args.filename)
        if not rv:
            sys.exit(1)


if __name__ == "__main__":
    main()
